require 'spec_helper'
require 'rees46'

describe 'Примеры использования' do
  let(:shop_id) { 'b6ef5e0003904ba8245eb7aac0c286' }
  let(:session_id) { '7b710da8-9b32-4a7b-ba2c-a8f14e635c98' }
  let(:user_id) { 46 }
  before(:each) { WebMock.allow_net_connect! }

  describe 'Инициализация' do
    context 'ID магазина' do
      it 'Необходимо передавать всегда' do
        REES46.configure do |c|
          c.shop_id = shop_id
        end

        expect(REES46.configuration.shop_id).to be_present
      end
    end

    context 'ID сессии' do
      context 'Когда нет' do
        it 'Модуль генерирует ID сессии' do
          REES46.configure do |c|
            c.shop_id = shop_id
          end

          expect(REES46.configuration.session_id).to be_a(String)
        end
      end

      context 'Когда есть' do
        it 'Необходимо передать ее в модуль' do
          REES46.configure do |c|
            c.shop_id = shop_id
            c.session_id = session_id
            c.user_id = user_id
          end

          expect(REES46.configuration.user_id).to be_present
        end
      end
    end

    context 'ID пользователя' do
      it 'Необходимо передавать в модуль' do
        REES46.configure do |c|
          c.shop_id = shop_id
          c.session_id = session_id
          c.user_id = user_id
        end
      end
    end
  end

  describe 'Отслеживание событий' do
    it 'Возможно только после инициализации модуля' do
      expect{
        REES46::Tracker.view(item: { item_id: 15 })
      }.to raise_error(REES46::NotConfiguredError)
    end

    context 'Типы событий' do
      before(:each) do
        REES46.configure do |c|
          c.shop_id = shop_id
          c.session_id = session_id
          c.user_id = user_id
        end
      end

      let(:item1) { { item_id: 5, price: 10_999.99, is_available: true, category: 15 } }
      let(:item2) { { item_id: 39, price: 458.10, is_available: true, categories: [4, 25] } }

      it 'Просмотр товара' do
        expect(REES46::Tracker.view(item: item1)).to eq({'status' => 'success'})
      end

      it 'Добавление товара в корзину' do
        expect(REES46::Tracker.cart(item: item1)).to eq({'status' => 'success'})
      end

      it 'Удаление товара из корзины' do
        expect(REES46::Tracker.remove_from_cart(item: item1)).to eq({'status' => 'success'})
      end

      it 'Покупка' do
        expect(REES46::Tracker.purchase(items: [item1.merge(amount: 2), item2.merge(amount: 5)], order_id: 91)).to eq({'status' => 'success'})
      end

      it 'Оценка' do
        expect(REES46::Tracker.rate(item: item1, rating: 3)).to eq({'status' => 'success'})
      end
    end
  end

  describe 'Запрос рекомендаций' do
    it 'Возможен только после инициализации модуля' do
      expect{
        REES46::Recommender.interesting
      }.to raise_error(REES46::NotConfiguredError)
    end

    context 'Виды рекомендаций' do
      before(:each) do
        REES46.configure do |c|
          c.shop_id = shop_id
          c.session_id = session_id
          c.user_id = user_id
        end
      end

      it 'Возможно, вас это заинтересует' do
        expect(REES46::Recommender.interesting).to be_an(Array)
      end

      it 'Популярные товары' do
        expect(REES46::Recommender.popular(category: 4)).to be_an(Array)
      end

      it 'Вы недавно смотрели' do
        expect(REES46::Recommender.recently_viewed).to be_an(Array)
      end

      it 'Похожие товары' do
        expect(REES46::Recommender.similar(item: 5)).to be_an(Array)
      end

      it 'С этим также покупают' do
        expect(REES46::Recommender.also_bought(item: 5)).to be_an(Array)
      end

      it 'Посмотрите также' do
        expect(REES46::Recommender.see_also(cart: [5, 39])).to be_an(Array)
      end

      it 'Прямо сейчас покупают' do
        expect(REES46::Recommender.buying_now).to be_an(Array)
      end
    end
  end
end
